'' =================================================================================================
''
''   File....... jm_can_demo.spin
''   Purpose.... Simple CAN sniffer/demo using MCP2515 interface
''   Author..... Jon "JonnyMac" McPhalen 
''               Copyright (c) 2011-12 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 16 JAN 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1      = 31                                                 ' debug/programming port
  TX1      = 30
  
  SDA      = 29                                                 ' eeprom/i2c buss
  SCL      = 28

' JonnyMac Controls Propeller Platform CAN Shield      
                                                                                  
  SPI_MISO = 27
  SPI_MOSI = 26
  SPI_SCK  = 25

  CAN_CS   = 24

' Propeller ASC with Arduino CAN Shield (www.fazjaxton.net)          
{
  SPI_SCK  = 13                                                                                   
  SPI_MISO = 12
  SPI_MOSI = 11

  CAN_CS   = 10 
}

con

  CAN_SPD  = 250                                                ' 250 kbits/sec                                                                                          


con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"
  can  : "jm_mcp2515"
 
  
var

  long  stack[16]                                               ' stack to timer/rtc cog

  word  sid                                                     ' can message standard id
  byte  dlc                                                     ' data length control
  byte  canbuf[8]                                               ' can msg data 

  byte  scs, mns, hrs, day                                      ' timer/rtc regs

                 
dat

Banner                  byte    "=======================", CR 
                        byte    " CAN Buss Sniffer/Demo ", CR
                        byte    "=======================", CR 
                        byte    CR
                        byte    0

Menu                    byte    "[H]ex", CR
                        byte    "[D]ecimal", CR
                        byte    CR
                        byte    0

Header                  byte    "RBn  SID  DLC  Data                           ", CR
                        byte    "---  ---  ---  -------------------------------", CR
                        byte    0

Pad                     byte    "  ", 0

  
pub main | last, style, ftype    

  setup                                                         ' start objects

  term.tx(CLS)
  term.str(@Banner)                                             ' setup display
  term.str(@Menu)

  repeat
    term.rxflush                                                ' clear keyboard buffer
    style := term.rx                                            ' wait for key
    case style
      "h", "H":                                                 ' hex?
        style := 16
        quit
      "d", "D":                                                 ' decimal?
        style := 10
        quit

  term.tx(CLS)      
  term.str(@Banner)       
  term.str(@Header)

  bytefill(@scs, 0, 3)                                          ' restart timer
  last := -1                                                    ' force initial display

  repeat
    if (last <> scs)                                            ' clock change?
      canbuf[0] := dec2bcd(scs)                                 ' convert to BCD
      canbuf[1] := dec2bcd(mns)
      canbuf[2] := dec2bcd(hrs)
      canbuf[3] := day                                          ' bcd conversion not needed
      can.tx_stdframe(0, $000, 4, @canbuf, can#PRI_HIGH)        ' send rtc to buss
      last := scs                                               ' save for next loop

                                                                                     
    ftype := (can.rx_stdframe(0, @sid, @dlc, @canbuf))          ' get frame type                       
    if (ftype)                                                  ' anything in RB0? 
      can.clear_rx_intf(0)                                      '  yes, release buffer
      if (ftype == can#FRAME_REMOTE)                            '  remote transfer request?
        canbuf[0] := dec2bcd(scs)                               '   yes, return seconds
        can.tx_stdframe(0, sid, 1, @canbuf, can#PRI_HIGH)       '   us rx'd sid
      display_frame(0, ftype, style)                            '  show contents of frame
      

    ftype := (can.rx_stdframe(1, @sid, @dlc, @canbuf))          ' check RB1          
    if (ftype)                                                
      can.clear_rx_intf(1)
      if (ftype == can#FRAME_REMOTE)                            
        canbuf[0] := dec2bcd(scs)                               
        can.tx_stdframe(0, sid, 1, @canbuf, can#PRI_HIGH)                                     
      display_frame(1, ftype, style)                            


pub display_frame(buf, type, nbase) | idx

'' Display (previously retrieved) CAN buffer data

  if ((buf <> 0) and (buf <> 1))
    return

  term.str(string("RB"))                                        ' show buffer
  term.bin(buf, 1)
  term.str(@Pad)                   
                                               
  term.hex(sid, 3)                                              ' standard id                             
  term.str(@Pad)                               
                                               
  term.rjdec(dlc, 3, " ")                                       ' dlc
  term.str(@Pad) 

  if (type == can#FRAME_DATA)                                   ' data frame?
    idx := 0
    repeat dlc                                                  ' display any data bytes
      if (nbase == 10)                   
        term.rjdec(canbuf[idx++], 3, " ")  
      else                               
        term.hex(canbuf[idx++], 2)         
      term.tx(" ")

  else
    term.str(string("Remote Transfer Request")) 

  term.tx(CR)                                  


con
                                                   
pub setup | check

'' Setup objects/hardware for CAN sniffer/demo

  term.start(RX1, TX1, %0000, 115_200)                          ' start terminal for PST
  cognew(rtc, @stack)                                           ' start timer/rtc cog     

  
  ' setup CAN interface

  can.start(CAN_CS, SPI_SCK, SPI_MOSI, SPI_MISO, -1)            
  can.reset

  can.set_mode(can#REQOP_CONFIG)                                ' set to configuration mode
  can.set_baud(CAN_SPD)                                         ' set buss speed
  can.write(can#TXB0CTRL, $00)                                  ' abort pending tx's
  can.write(can#TXB1CTRL, $00)
  can.write(can#TXB2CTRL, $00)
  can.write(can#TXRTSCTRL, $00)                                 ' set tx rts pins to inputs
  can.write(can#CANINTE, $00)                                   ' disable external interrupts
  can.write(can#CANINTF, $00)                                   ' clear interrupt flags  
  can.write(can#BFPCTRL, $00)                                   ' disable RXnBF pins
  
  can.set_rxb0_filters($000, $000, $000)                        ' set rx filters
  can.set_rxb1_filters($000, $000, $000, $000, $000)
  
  can.set_mode(can#REQOP_NORMAL)                                ' ready to rock!


pub dec2bcd(dec)

'' Convert decimal to BCD

  return ((dec / 10) << 4) | (dec // 10)
 

con

pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con
 
pri rtc | t                                                     ' start with cognew

'' Simple 24-hour RTC
'' -- runs in separate Spin cog 

  t := cnt
  repeat  
    waitcnt(t += clkfreq)                                      ' update every second
    scs := ++scs // 60
    if (scs == 0)
      mns := ++mns // 60
      if (mns == 0)
        hrs := ++hrs // 24
        if (hrs == 0)
          day := ++ day // 7

  
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}